package de.cliffbreak.varo.uitls;

import java.io.File;
import java.io.IOException;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.inventory.MainHand;
import org.bukkit.scheduler.BukkitRunnable;

import de.cliffbreak.varo.Varo;

public class PlayerCache {

    private final Varo plugin;
    private final File playerCacheFile;
    private final FileConfiguration playerCache;
    private boolean wasSaved = false;

    public PlayerCache(final Varo plugin) {
        this.playerCacheFile = new File("plugins/CliffbreakVaro", "playercache.yml");
        this.playerCache = YamlConfiguration.loadConfiguration(this.playerCacheFile);
        this.plugin = plugin;
    }

    public void setPlayerCache(final String uuid, final PlayerCacheType type, final Object value) {
        this.playerCache.set("Players." + uuid + "." + type.toString(), value);
        savePlayerCache();
    }

    public String getName(final String uuid) {
        return (String) this.playerCache.get("Players." + uuid + "." + PlayerCacheType.NAME.toString());
    }

    public int getSkinParts(final String uuid) {
        return (int) this.playerCache.get("Players." + uuid + "." + PlayerCacheType.SKINPARTS.toString());
    }

    public MainHand getMainHand(final String uuid) {
        return MainHand.valueOf(
                this.playerCache.get("Players." + uuid + "." + PlayerCacheType.MAINHAND.toString()).toString());
    }

    public byte getMainHandAsByte(final String uuid) {
        if (getMainHand(uuid) == MainHand.RIGHT)
            return (byte) 1;
        else
            return (byte) 0;
    }

    public void shutdown() {
        try {
            this.plugin.getLogger().info("Saving PlayerCache...");
            this.playerCache.save(this.playerCacheFile);
        } catch (final IOException e) {
            e.printStackTrace();
        }
    }

    private void savePlayerCache() {
        if (!this.wasSaved)
            try {
                this.plugin.getLogger().info("Saving PlayerCache...");
                this.playerCache.save(this.playerCacheFile);
                this.wasSaved = true;
                new BukkitRunnable() {

                    @Override
                    public void run() {
                        wasSaved = false;
                    }
                }.runTaskLater(this.plugin, 20); // Only save once every second to prevent disklock
            } catch (final IOException e) {
                e.printStackTrace();
            }
    }

    public enum PlayerCacheType {
        NAME, SKINPARTS, MAINHAND,
    }
}